@@ -45,7 +45,7 @@
 #include "nfx.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
+//#include "nfxstat.h"
 #include "nf_common.h"
 #include "rbtree.h"
 #include "nftree.h"
@@ -204,19 +204,19 @@ static int inline lnf_field_fset_TCP_FLAGS(lnf_rec_t *rec, void *p) {
 // however the structures are compatible so we will pretend 
 // that v6.srcaddr and v6.dst addr points to same structure 
 static int inline lnf_field_fget_SRCADDR(lnf_rec_t *rec, void *p) { 
-	ip_addr_t *d = (ip_addr_t *)&MR->v6.srcaddr;
+	ip_addr_t *d = (ip_addr_t *)&MR->V6.srcaddr;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return LNF_OK;
 }
 
 static int inline lnf_field_fset_SRCADDR(lnf_rec_t *rec, void *p) { 
-	ip_addr_t *d = (ip_addr_t *)&MR->v6.srcaddr;
+	ip_addr_t *d = (ip_addr_t *)&MR->V6.srcaddr;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		ClearFlag(MR->flags, FLAG_IPV6_ADDR);
@@ -229,19 +229,19 @@ static int inline lnf_field_fset_SRCADDR(lnf_rec_t *rec, void *p) {
 
 /* ----------------------- */
 static int inline lnf_field_fget_DSTADDR(lnf_rec_t *rec, void *p) { 
-	ip_addr_t *d = (ip_addr_t *)&MR->v6.dstaddr;
+	ip_addr_t *d = (ip_addr_t *)&MR->V6.dstaddr;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return LNF_OK;
 }
 
 static int inline lnf_field_fset_DSTADDR(lnf_rec_t *rec, void *p) { 
-	ip_addr_t *d = (ip_addr_t *)&MR->v6.dstaddr;
+	ip_addr_t *d = (ip_addr_t *)&MR->V6.dstaddr;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		ClearFlag(MR->flags, FLAG_IPV6_ADDR);
@@ -256,8 +256,8 @@ static int inline lnf_field_fset_DSTADDR(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fget_IP_NEXTHOP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = (ip_addr_t *)&MR->ip_nexthop;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return __bit_array_get(EA, EX_NEXT_HOP_v4) || __bit_array_get(EA, EX_NEXT_HOP_v6) ? LNF_OK : LNF_ERR_NOTSET;
 }
@@ -265,8 +265,8 @@ static int inline lnf_field_fget_IP_NEXTHOP(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fset_IP_NEXTHOP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = &MR->ip_nexthop;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		ClearFlag(MR->flags, FLAG_IPV6_NH);
@@ -380,8 +380,8 @@ static int inline lnf_field_fset_BGPPREVADJACENTAS(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fget_BGP_NEXTHOP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = (ip_addr_t *)&MR->bgp_nexthop;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return __bit_array_get(EA, EX_NEXT_HOP_BGP_v4) || __bit_array_get(EA, EX_NEXT_HOP_BGP_v6) ? LNF_OK : LNF_ERR_NOTSET;
 }
@@ -389,8 +389,8 @@ static int inline lnf_field_fget_BGP_NEXTHOP(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fset_BGP_NEXTHOP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = &MR->bgp_nexthop;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		ClearFlag(MR->flags, FLAG_IPV6_NHB);
@@ -578,8 +578,8 @@ static int inline lnf_field_fset_FWD_STATUS(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fget_IP_ROUTER(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = (ip_addr_t *)&MR->ip_router;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return __bit_array_get(EA, EX_ROUTER_IP_v4) || __bit_array_get(EA, EX_ROUTER_IP_v6) ? LNF_OK : LNF_ERR_NOTSET;
 }
@@ -587,8 +587,8 @@ static int inline lnf_field_fget_IP_ROUTER(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fset_IP_ROUTER(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = &MR->ip_router;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		ClearFlag(MR->flags, FLAG_IPV6_EXP);
@@ -705,8 +705,8 @@ static int inline lnf_field_fset_FW_EVENT(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fget_XLATE_SRC_IP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = (ip_addr_t *)&MR->xlate_src_ip;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return __bit_array_get(EA, EX_NSEL_XLATE_IP_v4) || __bit_array_get(EA, EX_NSEL_XLATE_IP_v6) ? LNF_OK : LNF_ERR_NOTSET;
 }
@@ -714,8 +714,8 @@ static int inline lnf_field_fget_XLATE_SRC_IP(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fset_XLATE_SRC_IP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = &MR->xlate_src_ip;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		__bit_array_set(EA,  EX_NSEL_XLATE_IP_v4, 1);
@@ -729,8 +729,8 @@ static int inline lnf_field_fset_XLATE_SRC_IP(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fget_XLATE_DST_IP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = (ip_addr_t *)&MR->xlate_dst_ip;
 	
-	((ip_addr_t *)p)->v6[0] = htonll(d->v6[0]);
-	((ip_addr_t *)p)->v6[1] = htonll(d->v6[1]);
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
 
 	return __bit_array_get(EA, EX_NSEL_XLATE_IP_v4) || __bit_array_get(EA, EX_NSEL_XLATE_IP_v6) ? LNF_OK : LNF_ERR_NOTSET;
 }
@@ -738,8 +738,8 @@ static int inline lnf_field_fget_XLATE_DST_IP(lnf_rec_t *rec, void *p) {
 static int inline lnf_field_fset_XLATE_DST_IP(lnf_rec_t *rec, void *p) { 
 	ip_addr_t *d = &MR->xlate_dst_ip;
 
-	d->v6[0] = ntohll( ((ip_addr_t *)p)->v6[0] );
-	d->v6[1] = ntohll( ((ip_addr_t *)p)->v6[1] );
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
 
 	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)p)) {
 		__bit_array_set(EA,  EX_NSEL_XLATE_IP_v4, 1);
@@ -1047,6 +1047,98 @@ static int inline lnf_field_fset_INET_FAMILY(lnf_rec_t *rec, void *p) {
 	}
 }
 
+/* ----------------------- */
+static int inline lnf_field_fget_EXPORTER_IP(lnf_rec_t *rec, void *p) { 
+	ip_addr_t *d = (ip_addr_t *)&rec->exporter->info.ip;
+	
+	((ip_addr_t *)p)->V6[0] = htonll(d->V6[0]);
+	((ip_addr_t *)p)->V6[1] = htonll(d->V6[1]);
+
+	return rec->flags & LNF_REC_EXPORTER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_EXPORTER_IP(lnf_rec_t *rec, void *p) { 
+	ip_addr_t *d = &rec->exporter->info.ip;
+
+	d->V6[0] = ntohll( ((ip_addr_t *)p)->V6[0] );
+	d->V6[1] = ntohll( ((ip_addr_t *)p)->V6[1] );
+
+	rec->flags |= LNF_REC_EXPORTER;
+	return LNF_OK;
+}
+
+/* ----------------------- */
+static int inline lnf_field_fget_EXPORTER_ID(lnf_rec_t *rec, void *p) { 
+	*((uint32_t *)p) = rec->exporter->info.id;
+	return rec->flags & LNF_REC_EXPORTER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_EXPORTER_ID(lnf_rec_t *rec, void *p) { 
+	rec->exporter->info.id = *((uint32_t *)p);
+	rec->flags |= LNF_REC_EXPORTER;
+	return LNF_OK;
+}
+
+/* ----------------------- */
+static int inline lnf_field_fget_EXPORTER_VERSION(lnf_rec_t *rec, void *p) { 
+	*((uint32_t *)p) = rec->exporter->info.version;
+	return rec->flags & LNF_REC_EXPORTER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_EXPORTER_VERSION(lnf_rec_t *rec, void *p) { 
+	rec->exporter->info.version = *((uint32_t *)p);
+	rec->flags |= LNF_REC_EXPORTER;
+	return LNF_OK;
+}
+
+/* ----------------------- */
+static int inline lnf_field_fget_SEQUENCE_FAILURES(lnf_rec_t *rec, void *p) { 
+	*((uint32_t *)p) = rec->sequence_failures;
+	return rec->flags & LNF_REC_EXPORTER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_SEQUENCE_FAILURES(lnf_rec_t *rec, void *p) { 
+	rec->sequence_failures = *((uint32_t *)p);
+	rec->flags |= LNF_REC_EXPORTER;
+	return LNF_OK;
+}
+
+/* ----------------------- */
+static int inline lnf_field_fget_SAMPLER_MODE(lnf_rec_t *rec, void *p) { 
+	*((uint16_t *)p) = rec->sampler->info.mode;
+	return rec->flags & LNF_REC_SAMPLER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_SAMPLER_MODE(lnf_rec_t *rec, void *p) { 
+	rec->sampler->info.mode = *((uint16_t *)p);
+	rec->flags |= LNF_REC_SAMPLER;
+	return LNF_OK;
+}
+
+/* ----------------------- */
+static int inline lnf_field_fget_SAMPLER_INTERVAL(lnf_rec_t *rec, void *p) { 
+	*((uint32_t *)p) = rec->sampler->info.interval;
+	return rec->flags & LNF_REC_SAMPLER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_SAMPLER_INTERVAL(lnf_rec_t *rec, void *p) { 
+	rec->sampler->info.interval = *((uint32_t *)p);
+	rec->flags |= LNF_REC_SAMPLER;
+	return LNF_OK;
+}
+
+/* ----------------------- */
+static int inline lnf_field_fget_SAMPLER_ID(lnf_rec_t *rec, void *p) { 
+	*((uint32_t *)p) = rec->sampler->info.id;
+	return rec->flags & LNF_REC_SAMPLER ? LNF_OK : LNF_ERR_NOTSET;
+}
+
+static int inline lnf_field_fset_SAMPLER_ID(lnf_rec_t *rec, void *p) { 
+	rec->sampler->info.id = *((uint32_t *)p);
+	rec->flags |= LNF_REC_SAMPLER;
+	return LNF_OK;
+}
+
 /* ----------------------- */
 static int inline lnf_field_fset_EMPTY_(lnf_rec_t *rec, void *p) { 
 	return LNF_OK;
@@ -1128,6 +1220,8 @@ static int inline lnf_field_fset_BREC1(lnf_rec_t *rec, void *p) {
 /* ----------------------- */
 
 /* text description of the fields */
+/* WARNING if you change something in here, make sure to also change LNF_FLD_xxx_ALIAS
+   if relevant, otherwise expect some inconsistent behaviour */
 lnf_field_def_t lnf_fields_def[] = {
 // pod:  =====================
 	[LNF_FLD_FIRST] = {
@@ -1177,12 +1271,22 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT64, sizeof(LNF_UINT64_T),			LNF_AGGR_SUM,	LNF_SORT_DESC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"pkts",		"The number of packets",
+		"packets",		"The number of packets",
 		"packetDeltaCount", 0, 2,
 		NULL, 0, 0, 
 		lnf_field_fget_DPKTS,
 		lnf_field_fset_DPKTS},
 
+    [LNF_FLD_DPKTS_ALIAS] = {
+        LNF_UINT64, sizeof(LNF_UINT64_T),			LNF_AGGR_SUM,	LNF_SORT_DESC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "pkts",		"The number of packets",
+        "packetDeltaCount", 0, 2,
+        NULL, 0, 0,
+        lnf_field_fget_DPKTS,
+        lnf_field_fset_DPKTS},
+
 	[LNF_FLD_OUT_BYTES] = {
 		LNF_UINT64, sizeof(LNF_UINT64_T),		LNF_AGGR_SUM,	LNF_SORT_DESC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
@@ -1197,18 +1301,28 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT64, sizeof(LNF_UINT64_T),		LNF_AGGR_SUM,	LNF_SORT_DESC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"outpkts",	"The number of output packets",
+		"outpackets",	"The number of output packets",
 		"postPacketDeltaCount", 0, 24,
 		NULL, 0, 0, 
 		lnf_field_fget_OUT_PKTS,
 		lnf_field_fset_OUT_PKTS},
 
+    [LNF_FLD_OUT_PKTS_ALIAS] = {
+        LNF_UINT64, sizeof(LNF_UINT64_T),		LNF_AGGR_SUM,	LNF_SORT_DESC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "outpkts",	"The number of output packets",
+        "postPacketDeltaCount", 0, 24,
+        NULL, 0, 0,
+        lnf_field_fget_OUT_PKTS,
+        lnf_field_fset_OUT_PKTS},
+
 	[LNF_FLD_AGGR_FLOWS] = {
-		LNF_UINT64, sizeof(LNF_UINT64_T),	LNF_AGGR_SUM,	LNF_SORT_DESC,	
+		LNF_UINT64, sizeof(LNF_UINT64_T),	LNF_AGGR_SUM,	LNF_SORT_DESC,
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		"flows",	"The number of flows (aggregated)",
-		"aggFlows", 0, 3,
+		"deltaFlowCount", 0, 3,
 		NULL, 0, 0, 
 // probably other elemt woul be more suitable
 		lnf_field_fget_AGGR_FLOWS,
@@ -1241,12 +1355,22 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_OR,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"tcpflags",		"TCP flags",
+		"flags",		"TCP flags",
 		"tcpControlBits", 0, 6,
 		NULL, 0, 0, 
 		lnf_field_fget_TCP_FLAGS,
 		lnf_field_fset_TCP_FLAGS},
 
+    [LNF_FLD_TCP_FLAGS_ALIAS] = {
+        LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_OR,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "tcpflags",		"TCP flags",
+        "tcpControlBits", 0, 6,
+        NULL, 0, 0,
+        lnf_field_fget_TCP_FLAGS,
+        lnf_field_fset_TCP_FLAGS},
+
 // pod}:
 // pod:  Layer 3 information
 // pod:  =====================
@@ -1270,16 +1394,46 @@ lnf_field_def_t lnf_fields_def[] = {
 		lnf_field_fget_DSTADDR,
 		lnf_field_fset_DSTADDR},
 
+    [LNF_FLD_SRCADDR_ALIAS] = {
+        LNF_ADDR, sizeof(LNF_ADDR_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "srcnet",	"Source IP address",
+        "sourceIPv4Address", 0, 8,
+        "sourceIPv6Address", 0, 27,
+        lnf_field_fget_SRCADDR,
+        lnf_field_fset_SRCADDR},
+
+    [LNF_FLD_DSTADDR_ALIAS] = {
+        LNF_ADDR, sizeof(LNF_ADDR_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "dstnet",	"Destination IP address",
+        "destinationIPv4Address", 0, 12,
+        "destinationIPv6Address", 0, 28,
+        lnf_field_fget_DSTADDR,
+        lnf_field_fset_DSTADDR},
+
 	[LNF_FLD_IP_NEXTHOP] = {
 		LNF_ADDR, sizeof(LNF_ADDR_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"nexthop",		"IP next hop",
+		"nextip",		"IP next hop",
 		"ipNextHopIPv4Address", 0, 15, 
 		"ipNextHopIPv6Address", 0, 62, 
 		lnf_field_fget_IP_NEXTHOP,
 		lnf_field_fset_IP_NEXTHOP},
 
+    [LNF_FLD_IP_NEXTHOP_ALIAS] = {
+        LNF_ADDR, sizeof(LNF_ADDR_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "nexthop",		"IP next hop",
+        "ipNextHopIPv4Address", 0, 15,
+        "ipNextHopIPv6Address", 0, 62,
+        lnf_field_fget_IP_NEXTHOP,
+        lnf_field_fset_IP_NEXTHOP},
+
 	[LNF_FLD_SRC_MASK] = {
 		LNF_UINT8, sizeof(LNF_UINT8_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
@@ -1505,17 +1659,27 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_ADDR, sizeof(LNF_ADDR_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"router",	"Exporting router IP",
+		"routerip",	"Exporting router IP",
 		"exporterIPv4Address", 0, 130, 
 		"exporterIPv6Address", 0, 131, 
 		lnf_field_fget_IP_ROUTER, 
-		lnf_field_fset_IP_ROUTER}, 
+		lnf_field_fset_IP_ROUTER},
+
+    [LNF_FLD_IP_ROUTER_ALIAS] = {
+        LNF_ADDR, sizeof(LNF_ADDR_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "router",	"Exporting router IP",
+        "exporterIPv4Address", 0, 130,
+        "exporterIPv6Address", 0, 131,
+        lnf_field_fget_IP_ROUTER,
+        lnf_field_fset_IP_ROUTER},
 
 	[LNF_FLD_ENGINE_TYPE] = {
 		LNF_UINT8, sizeof(LNF_UINT8_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"systype",	"Type of exporter",
+		"engine-type",	"Type of exporter",
 		"engineType", 0, 38, 
 		NULL, 0, 0, 
 		lnf_field_fget_ENGINE_TYPE,
@@ -1525,12 +1689,32 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"sysid",	"Internal SysID of exporter",
+		"engine-id",	"Internal SysID of exporter",
 		"engineId", 0, 39, 
 		NULL, 0, 0, 
 		lnf_field_fget_ENGINE_ID,
 		lnf_field_fset_ENGINE_ID},
 
+    [LNF_FLD_ENGINE_TYPE_ALIAS] = {
+        LNF_UINT8, sizeof(LNF_UINT8_T),	LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "systype",	"Type of exporter",
+        "engineType", 0, 38,
+        NULL, 0, 0,
+        lnf_field_fget_ENGINE_TYPE,
+        lnf_field_fset_ENGINE_TYPE},
+
+    [LNF_FLD_ENGINE_ID_ALIAS] = {
+        LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "sysid",	"Internal SysID of exporter",
+        "engineId", 0, 39,
+        NULL, 0, 0,
+        lnf_field_fget_ENGINE_ID,
+        lnf_field_fset_ENGINE_ID},
+
 // pod:
 // pod:  NSEL fields, see: http://www.cisco.com/en/US/docs/security/asa/asa81/netflow/netflow.html
 // pod:  =====================
@@ -1558,7 +1742,7 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"icmpcode",		"NSEL ICMP code value",
+		"icmp-code",		"NSEL ICMP code value",
 		"icmpCodeIPv4", 0, 177, 
 		"icmpCodeIPv6", 0, 179, 
 		lnf_field_fget_ICMP_CODE,
@@ -1568,12 +1752,32 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"icmptype",		"NSEL ICMP type value",
+		"icmp-type",		"NSEL ICMP type value",
 		"icmpTypeIPv4", 0, 176, 
 		"icmpTypeIPv6", 0, 178, 
 		lnf_field_fget_ICMP_TYPE,
 		lnf_field_fset_ICMP_TYPE},
 
+    [LNF_FLD_ICMP_CODE_ALIAS] = {
+        LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "icmpcode",		"NSEL ICMP code value",
+        "icmpCodeIPv4", 0, 177,
+        "icmpCodeIPv6", 0, 179,
+        lnf_field_fget_ICMP_CODE,
+        lnf_field_fset_ICMP_CODE},
+
+    [LNF_FLD_ICMP_TYPE_ALIAS] = {
+        LNF_UINT8, sizeof(LNF_UINT8_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        "icmptype",		"NSEL ICMP type value",
+        "icmpTypeIPv4", 0, 176,
+        "icmpTypeIPv6", 0, 178,
+        lnf_field_fget_ICMP_TYPE,
+        lnf_field_fset_ICMP_TYPE},
+
 	[LNF_FLD_FW_XEVENT] = {
 		LNF_UINT16, sizeof(LNF_UINT16_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
@@ -1838,11 +2042,74 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"inetfamily",	"IENT family for src/dst IP address (ipv4 or ipv6); platform dependend",
-		NULL, 0, 0, 
+		"inetfamily",	"IENT family for src/dst IP address (ipv4 or ipv6); platform dependant",
+		"ipVersion", 0, 60,
 		NULL, 0, 0, 
 		lnf_field_fget_INET_FAMILY,
 		lnf_field_fset_INET_FAMILY},
+	[LNF_FLD_EXPORTER_IP] = {
+		LNF_ADDR, sizeof(LNF_ADDR_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"exporterid",	"Exporter IP address",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_EXPORTER_IP,
+		lnf_field_fset_EXPORTER_IP},
+	[LNF_FLD_EXPORTER_ID] = {
+		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"exporterid",	"Exporter Observation Domain ID",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_EXPORTER_ID,
+		lnf_field_fset_EXPORTER_ID},
+	[LNF_FLD_EXPORTER_VERSION] = {
+		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"exporterversion",	"Version of exporter",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_EXPORTER_VERSION,
+		lnf_field_fset_EXPORTER_VERSION},
+	[LNF_FLD_SEQUENCE_FAILURES] = {
+		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_SUM,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"exporterversion",	"Naumber of sequence failures of data received from exporter",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_SEQUENCE_FAILURES,
+		lnf_field_fset_SEQUENCE_FAILURES},
+	[LNF_FLD_SAMPLER_MODE] = {
+		LNF_UINT16, sizeof(LNF_UINT16_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"samplermode",	"Sampling mode",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_SAMPLER_MODE,
+		lnf_field_fset_SAMPLER_MODE},
+	[LNF_FLD_SAMPLER_INTERVAL] = {
+		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"sampleinterval",	"Ssampling interval",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_SAMPLER_INTERVAL,
+		lnf_field_fset_SAMPLER_INTERVAL},
+	[LNF_FLD_SAMPLER_ID] = {
+		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_KEY,	LNF_SORT_ASC,	
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+		"samplerid",	"Sampler ID assigned by exporting device",
+		NULL, 0, 0, 
+		NULL, 0, 0, 
+		lnf_field_fget_SAMPLER_ID,
+		lnf_field_fset_SAMPLER_ID},
 // pod:
 // pod:  Calculated items
 // pod:  =====================
@@ -1850,7 +2117,7 @@ lnf_field_def_t lnf_fields_def[] = {
 		LNF_UINT64, sizeof(LNF_UINT64_T),		LNF_AGGR_SUM,	LNF_SORT_ASC,	
 		{LNF_FLD_FIRST, LNF_FLD_LAST, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_},
-		"duration",	"Flow duration (in miliseconds)",
+		"duration",	"Flow duration (in milliseconds)",
 		NULL, 0, 0, 
 		NULL, 0, 0, 
 		lnf_field_fget_CALC_DURATION,
@@ -1922,6 +2189,16 @@ lnf_field_def_t lnf_fields_def[] = {
 		lnf_field_fget_SRCADDR,
 		lnf_field_fset_SRCADDR},
 
+    [LNF_FLD_PAIR_ADDR_ALIAS] = {
+        LNF_ADDR, sizeof(LNF_ADDR_T),		LNF_AGGR_KEY,	LNF_SORT_NONE,
+        {LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
+        {LNF_FLD_SRCADDR, LNF_FLD_DSTADDR},
+        "net",		"Source or destination ip address (pair field)",
+        NULL, 0, 0,
+        NULL, 0, 0,
+        lnf_field_fget_SRCADDR,
+        lnf_field_fset_SRCADDR},
+
 	[LNF_FLD_PAIR_AS] = {
 		LNF_UINT32, sizeof(LNF_UINT32_T),		LNF_AGGR_KEY,	LNF_SORT_NONE,	
 		{LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_, LNF_FLD_ZERO_},
@@ -1964,7 +2241,6 @@ lnf_field_def_t lnf_fields_def[] = {
 		NULL}
 };
 
-
 /* return info information for fiels 
 */
 int lnf_fld_info(int field, int info, void *data, size_t size) {
