@@ -44,16 +44,16 @@
 #include <stdint.h>
 #endif
 
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
 #include "nfnet.h"
 #include "bookkeeper.h"
-#include "nfxstat.h"
-#include "nf_common.h"
+//#include "nfxstat.h" commented out for >= 1.26
+//#include "nfdump.h"
 #include "rbtree.h"
 #include "nftree.h"
 #include "nfprof.h"
-#include "nfdump.h"
 #include "nflowcache.h"
 #include "nfstat.h"
 #include "nfexport.h"
@@ -303,6 +303,22 @@ int lnf_open(lnf_file_t **lnf_filep, const char * filename, unsigned int flags,
 								flags & LNF_ANON, (char *)ident);
 
 	} else {
+		/* set file name in LOOP mode */
+		if (flags & LNF_READ_LOOP) {
+			if (filename == NULL) {
+				free(lnf_file);
+				return LNF_ERR_OTHER;;
+			}
+
+			lnf_file->filename = malloc(strlen(filename) + 1);
+			if (lnf_file->filename == NULL) {
+				free(lnf_file);
+				return LNF_ERR_OTHER;;
+			}
+
+			strcpy(lnf_file->filename, filename);
+		
+		}
 
 		lnf_file->nffile = OpenFile((char *)filename, NULL);
 
@@ -327,6 +343,10 @@ int lnf_open(lnf_file_t **lnf_filep, const char * filename, unsigned int flags,
 
 	lnf_file->v1convert_buffer = NULL;
 	lnf_file->lnf_map_list = NULL;
+	lnf_file->exporters = NULL;
+	lnf_file->samplers = NULL;
+
+	lnf_file->num_exporters = 0; 
 
 	i = 1;
 	lnf_file->max_num_extensions = 0;
@@ -337,6 +357,7 @@ int lnf_open(lnf_file_t **lnf_filep, const char * filename, unsigned int flags,
     pthread_mutex_unlock(&lnf_nfdump_filter_mutex);
 #endif
 
+
 	*lnf_filep = lnf_file;
 
 	return LNF_OK;
@@ -405,7 +426,9 @@ int lnf_info(lnf_file_t *lnf_file, int info, void *data, size_t size) {
 			reqsize = sizeof(int);
 			break;
 		case LNF_INFO_CATALOG: 
-			*((int *)buf) = h->flags & FLAG_CATALOG; 
+			/* removed in 1.6.18 - we will return 0 */
+			// *((int *)buf) = h->flags & FLAG_CATALOG; 
+			*((int *)buf) = 0;
 			reqsize = sizeof(int);
 			break;
 		case LNF_INFO_IDENT: 
@@ -523,10 +546,54 @@ int lnf_info(lnf_file_t *lnf_file, int info, void *data, size_t size) {
 }
 
 
+/* XXX should be redesigned */
+void lnf_update_exporter_stats(lnf_file_t *lnf_file, nffile_t *nffile) {
+
+	generic_exporter_t *exporter; 
+	exporter_stats_record_t *estats;
+	size_t size;
+	int i = 0;
+
+	size = sizeof(exporter_stats_record_t) + (lnf_file->num_exporters - 1 ) * sizeof(struct exporter_stat_s);
+	estats = malloc(size);
+
+	if (estats == NULL) {
+		return; 
+	}
+
+	memset(estats, 0x0, size); 
+
+	/* prepare header */
+	estats->header.type = ExporterStatRecordType;
+	estats->header.size = size;
+	estats->stat_count = lnf_file->num_exporters;
+
+	/* fill statistics */
+	exporter = lnf_file->exporters; 
+	while (exporter != NULL) {
+	
+		estats->stat[i].sysid = exporter->info.sysid; 	
+		estats->stat[i].sequence_failure = exporter->sequence_failure; 	
+		estats->stat[i].packets = exporter->packets; 	
+		estats->stat[i].flows = exporter->flows; 	
+		i++;
+
+		exporter = exporter->next;
+	}
+
+	AppendToBuffer(lnf_file->nffile, (void *)estats, estats->header.size);
+
+	free(estats);
+
+}
+
 /* close file handler and release related structures */
 void lnf_close(lnf_file_t *lnf_file) {
 
 	lnf_map_list_t *map_list, *tmp_map_list;
+	generic_exporter_t *exporter; 
+	generic_sampler_t *sampler;
+	void *tmp; 
 
 	if (lnf_file == NULL || lnf_file->nffile == NULL) {
 		return ;
@@ -534,6 +601,9 @@ void lnf_close(lnf_file_t *lnf_file) {
 
 	if (lnf_file->flags & LNF_WRITE) {
 
+		/* append exporter statistics into file */
+		lnf_update_exporter_stats(lnf_file, lnf_file->nffile);
+
 		// write the last records in buffer
 		if (lnf_file->nffile->block_header->NumRecords ) {
 			if ( WriteBlock(lnf_file->nffile) <= 0 ) {
@@ -566,12 +636,28 @@ void lnf_close(lnf_file_t *lnf_file) {
 		free(lnf_file->v1convert_buffer);
 	}
 
+	/* free exporter list */
+	exporter = lnf_file->exporters;
+	while (exporter != NULL) {
+		tmp = exporter;
+		exporter = exporter->next;	
+		free(tmp);
+	}
+
+	/* free sampler list */
+	sampler = lnf_file->samplers;
+	while (sampler != NULL) {
+		tmp = sampler;
+		sampler = sampler->next;	
+		free(tmp);
+	}
+
 	free(lnf_file);
 }
 
 /* return next record in file */
 /* status of read and fill pre-prepared structure lnf_rec */
-int lnf_read(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec) {
+int lnf_read_record(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec) {
 
 //master_record_t	*master_record;
 int ret;
@@ -652,18 +738,19 @@ begin:
 				FLOW_RECORD_NEXT(lnf_file->flow_record);	
 				goto begin;
 				break;
-			
+		
+		/* support for V0Type removed in 1.6.18 */	
 		case CommonRecordV0Type:
 
 				// convert common record v0
-				if ( lnf_file->v1convert_buffer == NULL) {
-					lnf_file->v1convert_buffer = malloc(65536); /* very suspisous part of code taken from nfdump */
-					if ( lnf_file->v1convert_buffer == NULL ) {
-						return LNF_ERR_NOMEM;
-					}
-				}
-				ConvertCommonV0((void *)lnf_file->flow_record, (common_record_t *)lnf_file->v1convert_buffer);
-				common_record_ptr = (common_record_t *)lnf_file->v1convert_buffer;
+//				if ( lnf_file->v1convert_buffer == NULL) {
+//					lnf_file->v1convert_buffer = malloc(65536); /* very suspisous part of code taken from nfdump */
+//					if ( lnf_file->v1convert_buffer == NULL ) {
+//						return LNF_ERR_NOMEM;
+//					}
+//				}
+//				ConvertCommonV0((void *)lnf_file->flow_record, (common_record_t *)lnf_file->v1convert_buffer);
+//				common_record_ptr = (common_record_t *)lnf_file->v1convert_buffer;
 
 				break;
 
@@ -727,6 +814,54 @@ begin:
 
 } /* end of _readfnction */
 
+/* return next record in file */
+int lnf_read(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec) {
+
+	int ret;
+	
+	if (lnf_file->flags & LNF_READ_LOOP) {
+NEW_READ:
+		ret = lnf_read_record(lnf_file, lnf_rec);
+
+		/* we are at the end of the file, we will wait for new records or inode change */
+		if (ret == LNF_EOF) {
+			struct stat stat_buf;
+			/* wait for while */
+			sleep(1);
+
+			/* check inode change -> indicastes new file */
+			if (stat(lnf_file->filename, &stat_buf) != 0) {
+				return LNF_EOF;
+			}
+
+			/* inode change - close odl file and open a new one */
+			if (lnf_file->inode != 0 && lnf_file->inode != stat_buf.st_ino) {
+
+				if (lnf_file->nffile != NULL) {
+					CloseFile(lnf_file->nffile);
+				}
+
+				lnf_file->nffile = OpenFile((char *)lnf_file->filename, NULL);
+
+				if (lnf_file->nffile == NULL) {
+					return LNF_ERR_OTHER;
+				}
+			}
+
+			lnf_file->inode = stat_buf.st_ino;
+
+			/* sleep for while and try to read again */
+			goto NEW_READ;
+
+		} else {
+			return ret;
+		}
+
+	} else {
+		/* normal read -> map function to lnf_read_file */
+		return lnf_read_record(lnf_file, lnf_rec);
+	}
+}
 
 extension_map_t * lnf_lookup_map(lnf_file_t *lnf_file, bit_array_t *ext ) {
 extension_map_t *map; 
@@ -815,14 +950,87 @@ int map_id = 0;
 	return map;
 }
 
+/* lookup exporter ID by exporter ID an eporter IP 
+fileds: LNF_FLD_EXPORTER_ID LNF_FLD_EXPORTER_IP 
+returns - internal exporter ID (sysid)
+
+Exporters are organized in linked lst. If the ID and IP is not 
+found in the list the new entry is created */
+generic_exporter_t* lnf_lookup_exporter(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec) {
+
+	generic_exporter_t *exporter;
+	generic_exporter_t *tmp;
+	ip_addr_t ip;
+
+	/* no exporter set in the record */
+	if ((lnf_rec->flags & LNF_REC_EXPORTER) == 0) {
+		return NULL;
+	}
+
+	exporter = lnf_file->exporters;
+
+	/* walk via all exporters */
+	while (exporter != NULL) {
+
+		if (lnf_rec->exporter->info.id == exporter->info.id && memcmp(&lnf_rec->exporter->info.ip, &exporter->info.ip, sizeof(ip_addr_t)) == 0) {
+			return exporter;
+		}
+
+		exporter = exporter->next;
+	}
+
+	exporter = calloc(sizeof(generic_exporter_t), 1);
+
+	if (exporter == NULL) {
+		return NULL;
+	}
+
+	/* add new exporter into linked list */
+	tmp = lnf_file->exporters;
+	lnf_file->exporters = exporter;
+	exporter->next = tmp;
+
+	/* copy exporter info data */
+	exporter->info.id = lnf_rec->exporter->info.id;
+	memcpy(&exporter->info.ip, &lnf_rec->exporter->info.ip, sizeof(ip_addr_t));
+
+	/* assign sysid */
+	lnf_file->num_exporters++;
+	//exporter->info.sysid = lnf_file->num_exporters - 1; /* numbering from exprters starts from 0 */
+	/* starting with v 1.6.18 the exporter sysid is counted from 1 */
+	exporter->info.sysid = lnf_file->num_exporters; /* numbering from exporters starts from 1 */
+
+	/* additional exporter_info_record_t fields */
+	exporter->info.version = lnf_rec->exporter->info.version; 
+
+	ip.V6[0] = htonll(exporter->info.ip.V6[0]);
+	ip.V6[1] = htonll(exporter->info.ip.V6[1]);
+
+	if (IN6_IS_ADDR_V4COMPAT((struct in6_addr *)&ip)) {
+		exporter->info.sa_family = AF_INET;
+	} else {
+		exporter->info.sa_family = AF_INET6;
+	}
+
+	exporter->info.header.size = sizeof(exporter_info_record_t);
+	exporter->info.header.type =  ExporterInfoRecordType;
+
+	AppendToBuffer(lnf_file->nffile, (void *)&exporter->info, exporter->info.header.size);
+
+
+	return exporter;
+}
+
 
 
 /* return next record in file */
 /* status of read and fill pre-prepared structure lnf_rec */
 int lnf_write(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec) {
-extension_map_t *map;
 
-	/* lookup and add map into file it it is nescessary */
+	extension_map_t *map;
+	generic_exporter_t *exporter;
+
+	/* lookup and add map into file if it it is nescessary */
 	map = lnf_lookup_map(lnf_file, lnf_rec->extensions_arr);
 
 	if (map == NULL) {
@@ -833,6 +1041,18 @@ extension_map_t *map;
 	lnf_rec->master_record->ext_map = map->map_id;
 	lnf_rec->master_record->type = CommonRecordType;
 
+	/* lookup and add exporter record into file if it is nescessary */
+	exporter = lnf_lookup_exporter(lnf_file, lnf_rec);
+
+	/* assign exporter sysid and update statistics */
+	if (exporter != NULL) {
+		lnf_rec->master_record->exporter_sysid = exporter->info.sysid;
+		exporter->packets += lnf_rec->master_record->dPkts;
+		exporter->flows += lnf_rec->master_record->aggr_flows;
+		exporter->sequence_failure += lnf_rec->sequence_failures;
+	}
+
+
 	UpdateStat(lnf_file->nffile->stat_record, lnf_rec->master_record);
 
 	PackRecord(lnf_rec->master_record, lnf_file->nffile);
@@ -874,6 +1094,22 @@ int lnf_rec_init(lnf_rec_t **recp) {
 	}
 
 
+	/* exporter and sampler structure initialisation */
+	rec->exporter = malloc(sizeof(generic_exporter_t));
+	rec->sampler = malloc(sizeof(generic_sampler_t));
+
+	if (rec->exporter == NULL || rec->sampler == NULL) {
+		lnf_rec_free(rec);
+		*recp = NULL;
+		return LNF_ERR_NOMEM;
+	}
+
+	memset(rec->exporter, 0x0, sizeof(generic_exporter_t));
+	memset(rec->sampler, 0x0, sizeof(generic_sampler_t));
+
+	rec->exporter->info.version = LNF_DEFAULT_EXPORTER_VERSION;
+
+
 	/* initialise nfdump extension list */
 	i = 1;
 	numext = 0;
@@ -921,7 +1157,13 @@ void lnf_rec_clear(lnf_rec_t *rec) {
 	if (rec != NULL) {
 		bit_array_clear(rec->extensions_arr);
 		memset(rec->master_record, 0x0, sizeof(master_record_t));
+		rec->sequence_failures = 0;
 	}
+
+	memset(rec->exporter, 0x0, sizeof(generic_exporter_t));
+	memset(rec->sampler, 0x0, sizeof(generic_sampler_t));
+
+	rec->exporter->info.version = LNF_DEFAULT_EXPORTER_VERSION;
 }
 
 /* copy record */
@@ -932,6 +1174,14 @@ int lnf_rec_copy(lnf_rec_t *dst, lnf_rec_t *src) {
 	}
 
 	memcpy(dst->master_record, src->master_record, sizeof(master_record_t));
+	memcpy(dst->exporter, src->exporter, sizeof(generic_exporter_t));
+	memcpy(dst->sampler, src->sampler, sizeof(generic_sampler_t));
+
+	dst->sequence_failures = src->sequence_failures;
+	dst->flags = src->flags;
+
+	dst->exporter->info.version = LNF_DEFAULT_EXPORTER_VERSION;
+
 	if ( bit_array_copy(dst->extensions_arr, src->extensions_arr)) {
 		return LNF_OK;
 	} else {
@@ -954,7 +1204,8 @@ int lnf_rec_get_raw(lnf_rec_t *rec, int type, char *buf, size_t size, size_t *re
 	}
 
 	if (type != LNF_REC_RAW_TLV) {
-		return LNF_ERR_OTHER;
+		lnf_seterror("%s: unsupported version in TLV (0x%x)", __func__, type);
+		return LNF_ERR_OTHER_MSG;
 	}
 
 	raw->version = LNF_REC_RAW_TLV;
@@ -1017,17 +1268,20 @@ int lnf_rec_set_raw(lnf_rec_t *rec, char *buf, size_t size) {
 
 	/* check size in buffer */
 	if (size < sizeof(lnf_rec_raw_t)) {
-		return LNF_ERR_OTHER;
+		lnf_seterror("%s: invalid buffer size");
+		return LNF_ERR_OTHER_MSG;
 	}
 
 	/* check version */
 	if (raw->version != LNF_REC_RAW_TLV) {
-		return LNF_ERR_OTHER;
+		lnf_seterror("%s: unsupported version in TLV (0x%x)", __func__, raw->version);
+		return LNF_ERR_OTHER_MSG;
 	}
 
 	/* check if we have all data in buffer */
-	if (raw->size + sizeof(lnf_rec_raw_t) < size) {
-		return LNF_ERR_OTHER;
+	if (raw->size + sizeof(lnf_rec_raw_t) > size) {
+		lnf_seterror("%s: the size of data (%dB) is slaller than buffer size (%dB)", __func__, raw->size + sizeof(lnf_rec_raw_t), size);
+		return LNF_ERR_OTHER_MSG;
 	}
 
 	lnf_rec_clear(rec);
@@ -1084,6 +1338,14 @@ void lnf_rec_free(lnf_rec_t *rec) {
 		free(rec->master_record);
 	}
 
+	if (rec->exporter != NULL) {
+		free(rec->exporter);
+	}
+
+	if (rec->sampler != NULL) {
+		free(rec->sampler);
+	}
+
 	if (rec->field_data != NULL) {
 		for (i = LNF_FLD_ZERO_; i < LNF_FLD_TERM_; i++) {
 			if (rec->field_data[i] != NULL) {
