@@ -42,6 +42,9 @@
 #include <rbtree.h>
 #include <nftree.h>
 #include <nfx.h>
+//#include <nfxstat.h> // commented out for >= 1.26
+#include <bookkeeper.h>
+#include <collector.h>
 #ifdef LNF_THREADS
 #include <pthread.h>
 #endif
@@ -60,6 +63,9 @@
 #define HAVE_HTONLL 1
 #endif
 
+#define LNF_DEFAULT_EXPORTER_VERSION 9
+
+
 /* list of maps used in file taht we create */
 typedef struct lnf_map_list_s {
 	bit_array_t				 bit_array;
@@ -91,6 +97,12 @@ typedef struct lnf_filter_s {
 typedef struct lnf_rec_s {
 	master_record_t *master_record;		/* reference to master record */
 	bit_array_t *extensions_arr;		/* list of extensions available in the record */
+	generic_exporter_t *exporter;		/* exporter information */
+	generic_sampler_t *sampler;			/* sampler information */
+#define LNF_REC_EXPORTER 0x1			/* is exporter set? */
+#define LNF_REC_SAMPLER 0x2				/* is sampler set ? */
+	int flags;
+	uint32_t sequence_failures;			/* sequence faulures for exporter information */
 	void **field_data;					/* list of pointers to data field */
 										/* data field is represented by pointer */
 										/* field_data[field] */
@@ -132,10 +144,18 @@ typedef struct lnf_file_s {
 	uint64_t                processed_blocks;
 	uint64_t                skipped_blocks;
 	uint64_t                processed_bytes;
+	char					*filename;				/* name of open file (for LOOP mode) */
+	ino_t					inode;					/* inode of open file (for LOOP mode) */
+	generic_exporter_t		*exporters;				/* linked list of exporters */
+	generic_sampler_t		*samplers;				/* linked list of samplers */
+	uint32_t				num_exporters;
 } lnf_file_t;
 
 
 extension_map_t * lnf_lookup_map(lnf_file_t *lnf_file, bit_array_t *ext );
+generic_exporter_t * lnf_lookup_exporter(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec);
+void lnf_update_exporter_stats(lnf_file_t *lnf_file, nffile_t *nffile);
+int lnf_read_record(lnf_file_t *lnf_file, lnf_rec_t *lnf_rec);
 
 /* nfdump uses LogError - we map it to lnf_seterror */
 void lnf_seterror(char *format, ...);
@@ -243,7 +263,69 @@ typedef struct lnf_fastaggr_s {
 
 /* filter operations - callbacks */
 ff_error_t lnf_ff_lookup_func(ff_t *filter, const char *fieldstr, ff_lvalue_t *lvalue);
-ff_error_t lnf_ff_data_func(ff_t *filter, void *rec, ff_extern_id_t id, char *data, size_t *size);
+ff_error_t lnf_ff_data_func(ff_t *filter, void *rec, ff_extern_id_t id, char **data, size_t *size);
+
+
+/****************************************************************/
+/* Ring buffer section                                          */
+/****************************************************************/
+
+#define LNF_MAX_RAW_LEN	1024			/* duplicated in libnf.h */
+#define LNF_MAX_STRING 512
+#define LNF_RINGBUF_SIZE 4096			/* number of items in ring buffer */
+#define LNF_RING_BLOCK_USLEEP 10		/* number of usesc to wait for next record when read is blocked */
+#define LNF_RING_STUCK_LIMIT  10000		/* wait for N cycles of LNF_RING_BLOCK_USLEEP to detect dead reader (100ms) */
+
+/* status of entry in ring buffer */
+typedef enum { 
+	LNF_RING_ENT_EMPTY = 0x0,	/* entry is empty */
+	LNF_RING_ENT_WRITE = 0x1,	/* data is being written to entry */
+	LNF_RING_ENT_READ = 0x2,	/* data is ready to read  */
+} lnf_ring_entry_status_t;
+
+
+/* representation of one entry in ring buffer */
+typedef struct lnf_ring_entry_s {
+
+	lnf_ring_entry_status_t status; 
+	int num_readers; 
+	long sequence; 
+	char* data[LNF_MAX_RAW_LEN + 1];
+	
+} lnf_ring_entry_t;
+
+
+/* structures in sharred memmory */
+typedef struct lnf_ring_shm_s {
+
+	pthread_mutex_t lock;			/* shm mutex */
+	long last_write_sequence;		/* last sequence number of written record */
+	int size;						/* number of record in ring buffer */
+	int conn_count;					/* number of connected instances */
+	int write_pos;					/* reference to first written record */
+	lnf_ring_entry_t entries[];		/* entires of ring buffer */
+
+} lnf_ring_shm_t;
+
+
+typedef struct lnf_ring_s {
+
+	long last_read_sequence; 		/* sequence of last read record */
+	int read_pos;			 		/* position of last read entry */
+	int fd;							/* file descriptor to shared file */
+	int blocking;					/* read is in blocking/ non blocking mode */
+	int force_release;				/* unlink shared mmerory - no matter how many process reads it */
+	uint64_t stuck_counter;				/* counter of stuck states */
+	uint64_t lost_counter;				/* counter of lost records */
+	uint64_t total_counter;				/* counter of total records */
+	char shm_name[LNF_MAX_STRING];	/* pointer to shared memmory area */
+	lnf_ring_shm_t *shm;			/* pointer to shared memmory area */
+	 		
+} lnf_ring_t;
+
+int lnf_ring_next(lnf_ring_t *ring, int index);
+int lnf_ring_lock(lnf_ring_t *ring);
 
 #endif /* _LIBNF_INTERNAL_H */
 
+
